/*
 * gSXprog by Eric Smith, based on sxprog by Loren Blaney
 *
 * Copyright 2001-2003 Eric Smith <eric@brouhaha.com>
 * Rev F SX-Key support added by Jim Garlick <garlick@llnl.gov>
 *
 * $Id: gsxprog.c,v 1.5 2003/11/18 13:59:42 garlick Exp $
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>            /* to get speed (B19200 etc.) */
#include <assert.h>
#include <libgen.h>
#include <getopt.h>

#include "gsxprog.h"
#include "sxkey.h"
#include "hex_file.h"
#include "sxkey_old.h"
#include "sxkey_new.h"

int debug_level = 0;
char *progname;

static void usage(void)
{
    fprintf(stderr, "Usage: %s [-d tty] [-r|-e|-v|-p|-R|-c MHz|-C|-f|-t|-i] [hexfile]\n",
            progname);
    fprintf(stderr,
  "-d --device tty  serial port device, e.g. /dev/ttyS0\n"
  "-D --debug       raise debug level\n"
  "-e --erase       erase SX device\n"
  "-r --read        read SX device (optionally to file)\n"
  "-p --program     program SX device from filename\n"
  "-v --verify      verify SX device against filename\n"
  "-R --reset       reset SX device\n"
  "-f --fuseinfo    read and display current fuse settings\n"
  "-f --trim        calibrate internal oscillator when programming\n"
  "-i --idskip      skip writing ID to reduce program time\n"
  "-c --clockon MHz generate clock to SX device (%-.3f - %-.3f MHz)\n"
  "-C --clockoff    do not generate clock\n", MIN_CLOCK, MAX_CLOCK);
    exit(1);
}

#define HAVE_GETOPT_LONG 1

#if HAVE_GETOPT_LONG
#define GETOPT(ac,av,opt,lopt) getopt_long(ac,av,opt,lopt,NULL)
#else
#define GETOPT(ac,av,opt,lopt) getopt(ac,av,opt)
#endif

#define OPTIONS "?Dd:revfpRc:Cti"
static struct option longopts[] = {
        {"debug", 0, 0, 'D'},
        {"device", 1, 0, 'd'},
        {"read", 0, 0, 'r'},
        {"erase", 0, 0, 'e'},
        {"verify", 0, 0, 'v'},
        {"fuseinfo", 0, 0, 'f'},
        {"program", 0, 0, 'p'},
        {"reset", 0, 0, 'R'},
        {"clockon", 1, 0, 'c'},
        {"clockoff", 0, 0, 'C'},
        {"trim", 0, 0, 't'},
        {"idskip", 0, 0, 'i'},
        {0, 0, 0, 0},
};

int main(int argc, char *argv[])
{
    enum {
        CMD_NONE     = 0x0000,
        CMD_READ     = 0x0001,
        CMD_ERASE    = 0x0002,
        CMD_PROG     = 0x0004,
        CMD_VERIFY   = 0x0008,
        CMD_RESET    = 0x0010,
        CMD_CLOCKON  = 0x0020,
        CMD_CLOCKOFF = 0x0040,
        CMD_FUSEINFO = 0x0080
    } cmd = CMD_NONE;
    int c;
    boolean r = false;
    char *hex_file_name = NULL;
    extern char *optarg;
    extern int optind;
    sx_t r_sx, w_sx;
    sxkey_t key = SXKEY_T_INIT;
    float frequency = 50.;
    boolean trimflag = false;
    boolean idskip = false;

    /*
     * Read parameters from environment, then override with command line.
     */
    {
        char *cp = getenv("SXKEY_DEV");

        if (cp != NULL)
            key.device = cp;
    }

    /*
     * Process command line args / environment.
     */
    progname = basename(argv[0]);
    while ((c = GETOPT(argc, argv, OPTIONS, longopts)) != EOF) {
        switch (c) {
        case 'D':              /* --debug */
            debug_level++;
            break;
        case 'R':              /* --reset */
            if (cmd != CMD_NONE)
                usage();
            cmd |= CMD_RESET;
            break;
        case 'd':              /* --device */
            key.device = optarg;
            break;
        case 'r':              /* --read */
            if (cmd != CMD_NONE)
                usage();
            cmd |= CMD_READ;
            break;
        case 'e':              /* --erase */
            if (cmd != CMD_NONE)
                usage();
            cmd |= CMD_ERASE;
            break;
        case 'v':              /* --verify */
            if (cmd != CMD_NONE)
                usage();
            cmd |= CMD_VERIFY;
            break;
        case 'f':              /* --fuseinfo */
            if ((cmd & ~(CMD_PROG|CMD_CLOCKON)) != CMD_NONE)
                usage();
            cmd |= CMD_FUSEINFO;
            break;
        case 'c':              /* --clockon */
            if ((cmd & ~(CMD_PROG|CMD_FUSEINFO)) != CMD_NONE)
                usage();
            cmd |= CMD_CLOCKON;
            frequency = atof( optarg );
            if(frequency < MIN_CLOCK || frequency > MAX_CLOCK) {
                fprintf(stderr, "%s: frequency out of range\n", progname);
                usage();
            }
            break;
        case 'C':              /* --clockoff */
            if (cmd != CMD_NONE)
                usage();
            cmd |= CMD_CLOCKOFF;
            break;
        case 'p':              /* --program */
            if ((cmd & ~(CMD_CLOCKON|CMD_FUSEINFO)) != CMD_NONE)
                usage();
            cmd |= CMD_PROG;
            break;
        case 't':              /* --trim */
            trimflag = true;
            break;
        case 'i':              /* --idskip */
            idskip = true;
            break;
        default:
            usage();
            break;
        }
    }
    if (optind < argc)
        hex_file_name = argv[optind++];
    if (optind < argc)
        usage();
    if (!hex_file_name && (cmd & CMD_VERIFY || cmd & CMD_PROG)) {
        fprintf(stderr, "%s: programming/verification requires a filename\n",
                progname);
        exit(1);
    }
    if ((trimflag || idskip) && !(cmd & CMD_PROG)) {
        fprintf(stderr, "%s: --idskip and --trim only apply during programming\n",
                progname);
        exit(1);
    }

    /*
     * Open serial port and send prolog.
     */
    if (!new_init(key.device, B57600, &key, &w_sx)
        && !new_init(key.device, B19200, &key, &w_sx)
        && !old_init(key.device, B19200, &key)) {
        fprintf(stderr,
                "%s: failed to establish communication with SX-Key\n",
                progname);
        exit(1);
    }

    /*
     * Detect target device type.
     */

    /*
     * Execute command.
     */
    r = true;

    if(cmd & CMD_ERASE) {
        r = key.cmd_erase(&key);
        if (!r) 
            goto finish;
    }
    if(cmd & CMD_READ) {
        r = key.cmd_read(&key, &r_sx);
        if (r) {
            if (hex_file_name)
                r = write_hex_file(hex_file_name, &r_sx);
            else
                sx_show(&r_sx);
        }
        if (!r) 
            goto finish;
    }
    if(cmd & CMD_PROG) {
        r = read_hex_file(hex_file_name, &w_sx);
        if (r)
            r = key.cmd_prog(&key, &w_sx, trimflag, idskip);
        if (!r) 
            goto finish;
    }
    if(cmd & CMD_VERIFY) {
        r = read_hex_file(hex_file_name, &w_sx);
        if (r) {
            r = key.cmd_read(&key, &r_sx);
            if (r)
                r = sx_compare(&w_sx, &r_sx, true, false);
        }
        if (!r) 
            goto finish;
    }
    if(cmd & CMD_FUSEINFO) {
        if (hex_file_name)
            r = read_hex_file(hex_file_name, &r_sx);
        else if (key.cmd_read_fuses)
            r = key.cmd_read_fuses(&key, &r_sx);
        else
            r = key.cmd_read(&key, &r_sx);
        if (r)
            sx_show_fuse_bits(&r_sx);
        if (!r) 
            goto finish;
    }
    if(cmd & CMD_CLOCKOFF) {
        if (key.cmd_clock_off)
            r = key.cmd_clock_off(&key);
        else {
            fprintf(stderr, "%s: clock_off not implemented\n", progname);
            r = false;
        }
        if (!r) 
            goto finish;
    }
    if(cmd & CMD_CLOCKON) {
        if (key.cmd_clock_on)
            r = key.cmd_clock_on(&key, frequency);
        else {
            fprintf(stderr, "%s: clock_on not implemented\n", progname);
            r = false;
        }
        if (!r) 
            goto finish;
    }
    if(cmd & CMD_RESET) {
        if (key.cmd_reset)
            r = key.cmd_reset(&key);
        else {
            fprintf(stderr, "%s: reset not implemented\n", progname);
            r = false;
        }
        if (!r) 
            goto finish;
    }

finish:
    /* 
     * Send epilog and close serial port.
     */
    key.fini(&key);

    exit(r ? 0 : 3);
}

/*
 * em:Local Variables: 
 * em:indent-tabs-mode:nil
 * em:c-basic-offset:4
 * em:End:
 * vi:tabstop=4 shiftwidth=4 expandtab
 */
